@@ -93,7 +93,7 @@ void SolidHwTest::testDeviceBasicFeatures()
     // Query properties
     QCOMPARE(valid_dev.as<Solid::GenericInterface>()->propertyExists("name"), true);
     QCOMPARE(valid_dev.as<Solid::GenericInterface>()->propertyExists("foo.bar"), false);
-    QCOMPARE((QObject *)invalid_dev.as<Solid::GenericInterface>(), (QObject *)0);
+    QCOMPARE((QObject *)invalid_dev.as<Solid::GenericInterface>(), (QObject *)nullptr);
 
     QCOMPARE(valid_dev.as<Solid::GenericInterface>()->property("name"), QVariant("Solid IDE DVD Writer"));
     QVERIFY(!valid_dev.as<Solid::GenericInterface>()->property("foo.bar").isValid());
@@ -229,7 +229,7 @@ void SolidHwTest::testDeviceInterfaces()
     Solid::DeviceInterface *processor = cpu.as<Solid::Processor>();
 
     QVERIFY(cpu.isDeviceInterface(Solid::DeviceInterface::Processor));
-    QVERIFY(iface != 0);
+    QVERIFY(iface != nullptr);
     QCOMPARE(iface, processor);
 
     Solid::Device cpu2("/org/kde/solid/fakehw/acpi_CPU0");
@@ -237,9 +237,9 @@ void SolidHwTest::testDeviceInterfaces()
     QCOMPARE(cpu.as<Solid::GenericInterface>(), cpu2.as<Solid::GenericInterface>());
 
     QPointer<Solid::Processor> p = cpu.as<Solid::Processor>();
-    QVERIFY(p != 0);
+    QVERIFY(p != nullptr);
     fakeManager->unplug("/org/kde/solid/fakehw/acpi_CPU0");
-    QVERIFY(p == 0);
+    QVERIFY(p == nullptr);
     fakeManager->plug("/org/kde/solid/fakehw/acpi_CPU0");
 
     QPointer<Solid::StorageVolume> v;
@@ -247,21 +247,21 @@ void SolidHwTest::testDeviceInterfaces()
     {
         Solid::Device partition("/org/kde/solid/fakehw/volume_uuid_f00ba7");
         v = partition.as<Solid::StorageVolume>();
-        QVERIFY(v != 0);
+        QVERIFY(v != nullptr);
         {
             Solid::Device partition2("/org/kde/solid/fakehw/volume_uuid_f00ba7");
             v2 = partition2.as<Solid::StorageVolume>();
-            QVERIFY(v2 != 0);
+            QVERIFY(v2 != nullptr);
             QVERIFY(v == v2);
         }
-        QVERIFY(v != 0);
-        QVERIFY(v2 != 0);
+        QVERIFY(v != nullptr);
+        QVERIFY(v2 != nullptr);
     }
-    QVERIFY(v != 0);
-    QVERIFY(v2 != 0);
+    QVERIFY(v != nullptr);
+    QVERIFY(v2 != nullptr);
     fakeManager->unplug("/org/kde/solid/fakehw/volume_uuid_f00ba7");
-    QVERIFY(v == 0);
-    QVERIFY(v2 == 0);
+    QVERIFY(v == nullptr);
+    QVERIFY(v2 == nullptr);
     fakeManager->plug("/org/kde/solid/fakehw/volume_uuid_f00ba7");
 }
 
@@ -297,17 +297,27 @@ void SolidHwTest::testDeviceInterfaceIntrospectionCornerCases()
     QCOMPARE((int)Solid::DeviceInterface::stringToType("blup"), -1);
 }
 
-static QSet<QString> to_string_set(const QList<Solid::Device> &list)
+static QStringList to_string_list(const QList<Solid::Device> &list)
 {
-    QSet<QString> res;
-
-    Q_FOREACH (const Solid::Device &device, list) {
+    QStringList res;
+    res.reserve(list.size());
+    for (const Solid::Device &device : list) {
         res << device.udi();
     }
-
     return res;
 }
 
+void SolidHwTest::testInvalidPredicate()
+{
+    QString str_pred = "[[Processor.maxSpeed == 3201 AND Processor.canChangeFrequency == false] OR StorageVolume.mountPoint == '/media/blup']";
+    // Since str_pred is canonicalized, fromString().toString() should be invariant
+    QCOMPARE(Solid::Predicate::fromString(str_pred).toString(), str_pred);
+
+    // Invalid predicate
+    str_pred = "[StorageVolume.ignored == false AND OpticalDisc.isBlank == true AND OpticalDisc.discType & 'CdRecordable|CdRewritable']";
+    QVERIFY(!Solid::Predicate::fromString(str_pred).isValid());
+}
+
 void SolidHwTest::testPredicate()
 {
     Solid::Device dev("/org/kde/solid/fakehw/acpi_CPU0");
@@ -335,14 +345,82 @@ void SolidHwTest::testPredicate()
     QVERIFY(p6.matches(dev));
     QVERIFY(p7.matches(dev));
 
-    QString str_pred = "[[Processor.maxSpeed == 3201 AND Processor.canChangeFrequency == false] OR StorageVolume.mountPoint == '/media/blup']";
-    // Since str_pred is canonicalized, fromString().toString() should be invariant
-    QCOMPARE(Solid::Predicate::fromString(str_pred).toString(), str_pred);
+    QList<Solid::Device> list;
 
-    // Invalid predicate
-    str_pred = "[StorageVolume.ignored == false AND OpticalDisc.isBlank == true AND OpticalDisc.discType & 'CdRecordable|CdRewritable']";
-    QVERIFY(!Solid::Predicate::fromString(str_pred).isValid());
+    QStringList cpuSet;
+    cpuSet << QString("/org/kde/solid/fakehw/acpi_CPU0")
+           << QString("/org/kde/solid/fakehw/acpi_CPU1");
 
+    list = Solid::Device::listFromQuery(p1);
+    QCOMPARE(list.size(), 2);
+    QCOMPARE(to_string_list(list), cpuSet);
+
+    list = Solid::Device::listFromQuery(p2);
+    QCOMPARE(list.size(), 0);
+
+    list = Solid::Device::listFromQuery(p3);
+    QCOMPARE(list.size(), 2);
+    QCOMPARE(to_string_list(list), cpuSet);
+
+    list = Solid::Device::listFromQuery(p4);
+    QCOMPARE(list.size(), 0);
+
+    list = Solid::Device::listFromQuery("[Processor.canChangeFrequency==true AND Processor.number==1]");
+    QCOMPARE(list.size(), 1);
+    QCOMPARE(list.at(0).udi(), QString("/org/kde/solid/fakehw/acpi_CPU1"));
+
+}
+
+void SolidHwTest::testQueryStorageVolumeOrProcessor()
+{
+    auto list = Solid::Device::listFromQuery("[Processor.number==1 OR IS StorageVolume]");
+    QCOMPARE(list.size(), 10);
+
+    //make sure predicate case-insensitiveness is sane
+    list = Solid::Device::listFromQuery("[Processor.number==1 or is StorageVolume]");
+    QCOMPARE(list.size(), 10);
+    list = Solid::Device::listFromQuery("[Processor.number==1 oR Is StorageVolume]");
+    QCOMPARE(list.size(), 10);
+    QStringList expected{
+        "/org/kde/solid/fakehw/acpi_CPU1",
+        "/org/kde/solid/fakehw/platform_floppy_0_storage_virt_volume",
+        "/org/kde/solid/fakehw/volume_label_SOLIDMAN_BEGINS",
+        "/org/kde/solid/fakehw/volume_part1_size_993284096",
+        "/org/kde/solid/fakehw/volume_part2_size_1024",
+        "/org/kde/solid/fakehw/volume_part5_size_1048576",
+        "/org/kde/solid/fakehw/volume_uuid_5011",
+        "/org/kde/solid/fakehw/volume_uuid_c0ffee",
+        "/org/kde/solid/fakehw/volume_uuid_f00ba7",
+        "/org/kde/solid/fakehw/volume_uuid_feedface"};
+    QCOMPARE(to_string_list(list), expected);
+
+    list = Solid::Device::listFromQuery("[IS Processor OR IS StorageVolume]");
+    QCOMPARE(list.size(), 11);
+    expected.prepend("/org/kde/solid/fakehw/acpi_CPU0");
+    QCOMPARE(to_string_list(list), expected);
+}
+
+void SolidHwTest::testQueryStorageVolumeOrStorageAccess()
+{
+    // the query from KFilePlacesModel
+    const auto list = Solid::Device::listFromQuery("[[[[ StorageVolume.ignored == false AND [ StorageVolume.usage == 'FileSystem' OR StorageVolume.usage == 'Encrypted' ]]"
+            " OR "
+            "[ IS StorageAccess AND StorageDrive.driveType == 'Floppy' ]]"
+            " OR "
+            "OpticalDisc.availableContent & 'Audio' ]"
+            " OR "
+            "StorageAccess.ignored == false ]");
+    const QStringList expected{
+            "/org/kde/solid/fakehw/fstab/thehost/solidpath",
+            "/org/kde/solid/fakehw/platform_floppy_0_storage_virt_volume",
+            "/org/kde/solid/fakehw/volume_part1_size_993284096",
+            "/org/kde/solid/fakehw/volume_uuid_5011",
+            "/org/kde/solid/fakehw/volume_uuid_f00ba7"};
+    QCOMPARE(to_string_list(list), expected);
+}
+
+void SolidHwTest::testQueryWithParentUdi()
+{
     QString parentUdi = "/org/kde/solid/fakehw/storage_model_solid_reader";
     Solid::DeviceInterface::Type ifaceType = Solid::DeviceInterface::Unknown;
     QCOMPARE(fakeManager->devicesFromQuery(parentUdi, ifaceType).size(), 1);
@@ -358,74 +436,20 @@ void SolidHwTest::testPredicate()
              QString("/org/kde/solid/fakehw/acpi_CPU0"));
     QCOMPARE(fakeManager->devicesFromQuery(parentUdi, ifaceType).at(1),
              QString("/org/kde/solid/fakehw/acpi_CPU1"));
+}
 
-    parentUdi.clear();
-    ifaceType = Solid::DeviceInterface::Unknown;
-    QList<Solid::Device> list;
-
-    QSet<QString> cpuSet;
-    cpuSet << QString("/org/kde/solid/fakehw/acpi_CPU0")
-           << QString("/org/kde/solid/fakehw/acpi_CPU1");
-
-    list = Solid::Device::listFromQuery(p1, parentUdi);
-    QCOMPARE(list.size(), 2);
-    QCOMPARE(to_string_set(list), cpuSet);
-
-    list = Solid::Device::listFromQuery(p2, parentUdi);
-    QCOMPARE(list.size(), 0);
-
-    list = Solid::Device::listFromQuery(p3, parentUdi);
-    QCOMPARE(list.size(), 2);
-    QCOMPARE(to_string_set(list), cpuSet);
-
-    list = Solid::Device::listFromQuery(p4, parentUdi);
-    QCOMPARE(list.size(), 0);
-
-    list = Solid::Device::listFromQuery("[Processor.canChangeFrequency==true AND Processor.number==1]",
-                                        parentUdi);
-    QCOMPARE(list.size(), 1);
-    QCOMPARE(list.at(0).udi(), QString("/org/kde/solid/fakehw/acpi_CPU1"));
-
-    list = Solid::Device::listFromQuery("[Processor.number==1 OR IS StorageVolume]",
-                                        parentUdi);
-    QCOMPARE(list.size(), 10);
-
-    //make sure predicate case-insensitiveness is sane
-    list = Solid::Device::listFromQuery("[Processor.number==1 or is StorageVolume]",
-                                        parentUdi);
-    QCOMPARE(list.size(), 10);
-    list = Solid::Device::listFromQuery("[Processor.number==1 oR Is StorageVolume]",
-                                        parentUdi);
-    QCOMPARE(list.size(), 10);
-
-    QSet<QString> set;
-    QCOMPARE(list.size(), 10);
-    set << "/org/kde/solid/fakehw/acpi_CPU1"
-        << "/org/kde/solid/fakehw/platform_floppy_0_storage_virt_volume"
-        << "/org/kde/solid/fakehw/volume_label_SOLIDMAN_BEGINS"
-        << "/org/kde/solid/fakehw/volume_part1_size_993284096"
-        << "/org/kde/solid/fakehw/volume_part2_size_1024"
-        << "/org/kde/solid/fakehw/volume_part5_size_1048576"
-        << "/org/kde/solid/fakehw/volume_uuid_5011"
-        << "/org/kde/solid/fakehw/volume_uuid_c0ffee"
-        << "/org/kde/solid/fakehw/volume_uuid_f00ba7"
-        << "/org/kde/solid/fakehw/volume_uuid_feedface";
-    QCOMPARE(set, to_string_set(list));
-
-    list = Solid::Device::listFromQuery("[IS Processor OR IS StorageVolume]",
-                                        parentUdi);
-    QCOMPARE(list.size(), 11);
-    set << "/org/kde/solid/fakehw/acpi_CPU0";
-    QCOMPARE(set, to_string_set(list));
-
-    ifaceType = Solid::DeviceInterface::Processor;
-    list = Solid::Device::listFromType(ifaceType, parentUdi);
+void SolidHwTest::testListFromTypeProcessor()
+{
+    const auto ifaceType = Solid::DeviceInterface::Processor;
+    const auto list = Solid::Device::listFromType(ifaceType, QString());
     QCOMPARE(list.size(), 2);
     QCOMPARE(list.at(0).udi(), QString("/org/kde/solid/fakehw/acpi_CPU0"));
     QCOMPARE(list.at(1).udi(), QString("/org/kde/solid/fakehw/acpi_CPU1"));
+}
 
-    ifaceType = Solid::DeviceInterface::Unknown;
-    list = Solid::Device::listFromQuery("blup", parentUdi);
+void SolidHwTest::testListFromTypeInvalid()
+{
+    const auto list = Solid::Device::listFromQuery("blup", QString());
     QCOMPARE(list.size(), 0);
 }
 
